package dev.peerat.framework;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

public class RouteCachable{
	
	private List<RouteInterceptor> interceptors;
	private Map<RouteFilter, List<RouteInterceptor>> localInterceptors;
	private Map<RouteFilter, Class<?>[]> types;
	
	public RouteCachable(){
		this.interceptors = new ArrayList<>();
		this.localInterceptors = new HashMap<>();
		this.types = new HashMap<>();
	}
	
	public void addInterceptor(RouteMapper<?>[] mappers, RouteInterceptor interceptor){
		this.interceptors.add(interceptor);
		for(RouteMapper<?> mapper : mappers) mapper.addInterceptor(interceptor);
	}
	
	public void addInterceptor(RouteMapper<?>[] mappers, RouteFilter filter, RouteInterceptor interceptor){
		List<RouteInterceptor> list = this.localInterceptors.get(filter);
		if(list == null){
			list = new ArrayList<>();
			this.localInterceptors.put(filter, list);
		}
		list.add(interceptor);
		
		for(RouteMapper<?> mapper : mappers) mapper.addInterceptor(filter, interceptor);
	}
	
	public void bind(RouteMapper<?>[] mappers, RouteFilter filter, Class<?>... types){
		Class<?>[] array = this.types.get(filter);
		Class<?>[] copy;
		if(array == null){
			copy = types;
		}else{
			copy = new Class[array.length+types.length];
			System.arraycopy(array, 0, copy, 0, array.length);
			System.arraycopy(types, 0, copy, array.length, types.length);
		}
		this.types.put(filter, copy);
		
		for(RouteMapper<?> mapper : mappers) mapper.bind(filter, types);
	}
	
	public void registerRoute(RouteState route){
		if(!this.interceptors.isEmpty()) route.addInterceptors(this.interceptors);
		for(Entry<RouteFilter, List<RouteInterceptor>> entry : this.localInterceptors.entrySet()){
			if(entry.getKey().accept(route.getMethod(), route.getRoute(), route.getPattern())) route.addInterceptors(entry.getValue());
		}
		for(Entry<RouteFilter, Class<?>[]> entry : this.types.entrySet()){
			if(entry.getKey().accept(route.getMethod(), route.getRoute(), route.getPattern())) route.getBinder().bind(entry.getValue());
		}
	}
}
