package dev.peerat.framework;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class RouteState {
	
	private Object instance;
	private Method method;
	private Route route;
	private Pattern pattern;
	private RouteBinder binder;
	private List<RouteInterceptor> interceptors;	//TODO concat interceptors ??
	
	public RouteState(Object instance, Method method, Route route){
		this.instance = instance;
		this.method = method;
		this.route = route;
		this.pattern = Pattern.compile(route.path());
		this.binder = new RouteBinder();
	}
	
	public void bindIndexes(){
		this.binder.bindIndexes(this.method);
	}
	
	public Matcher match(String path){
		Matcher matcher = pattern.matcher(path);
		return matcher.matches() ? matcher : null;
	}
	
	public void addInterceptor(RouteInterceptor interceptor){
		if(this.interceptors == null) this.interceptors = new ArrayList<>(); 
		this.interceptors.add(interceptor);
	}
	
	public void addInterceptors(Collection<RouteInterceptor> interceptors){
		if(this.interceptors == null) this.interceptors = new ArrayList<>(interceptors); 
		else this.interceptors.addAll(interceptors);
	}
	
	public List<RouteInterceptor> getInterceptors(){
		return this.interceptors;
	}
	
	public Route getRoute(){
		return this.route;
	}
	
	public boolean isWebSocket(){
		return this.route.websocket();
	}
	
	public boolean needLogin(){
		return this.route.needLogin();
	}
	
	public Method getMethod(){
		return this.method;
	}
	
	public Object getInstance(){
		return this.instance;
	}
	
	public Pattern getPattern(){
		return this.pattern;
	}
	
	public RouteBinder getBinder(){
		return this.binder;
	}
	
}
